@@ -9,11 +9,12 @@ import os
 
 from rlpython.python_runtime import PythonRuntime
 from rlpython.shell_runtime import ShellRuntime
-from rlpython.utils.color import color
+from rlpython.utils.strings import color
 from rlpython import VERSION_STRING
 
 DEFAULT_HISTORY_FILE = '~/.rlpython.history'
 DEFAULT_HISTORY_SIZE = 1000
+DEFAULT_PROMPT_PREFIX = ''
 DEFAULT_PROMPT = '>>> '
 DEFAULT_PROMPT_PS2 = '... '
 
@@ -27,7 +28,13 @@ rlpython {}
 
 
 class Repl:
+    class DOMAIN:
+        LOCAL = 1
+        NETWORK = 2
+        LOCAL_NETWORK = 3
+
     def __init__(self, banner=DEFAULT_BANNER, warnings=[],
+                 prompt_prefix=DEFAULT_PROMPT_PREFIX,
                  prompt=DEFAULT_PROMPT,
                  prompt_ps2=DEFAULT_PROMPT_PS2,
                  history_file=DEFAULT_HISTORY_FILE,
@@ -36,6 +43,7 @@ class Repl:
 
         self.banner = banner
         self.warnings = warnings
+        self.prompt_prefix = prompt_prefix
         self.ps1 = prompt
         self.ps2 = prompt_ps2
         self.history_file = os.path.expanduser(history_file)
@@ -65,6 +73,24 @@ class Repl:
         self.clear_line_buffer()
         self.setup()
 
+    def shutdown(self):
+        self.python_runtime.shutdown()
+        self.shell_runtime.shutdown()
+
+        if self.domain == self.DOMAIN.LOCAL:
+            self.write_history()
+
+    # domain ##################################################################
+    @property
+    def domain(self):
+        if not hasattr(self, '_domain'):
+            self._domain = self.DOMAIN.LOCAL
+
+        return self._domain
+
+    def set_domain(self, domain):
+        self._domain = domain
+
     # history #################################################################
     def read_history(self):
         self.history = []
@@ -118,40 +144,48 @@ class Repl:
 
             text = '\n'.join(lines) + '\n'
 
-        self.write(color(text, color='red'))
+        self.write(color(text, fg='red'))
 
     # line buffer #############################################################
     def clear_line_buffer(self):
         self.line_buffer = ''
 
-    def _validate_line_buffer(self):
+    def validate_line_buffer(self):
         if self.line_buffer.startswith('%'):
             return self.shell_runtime.validate_source(self.line_buffer)
 
         return self.python_runtime.validate_source(self.line_buffer)
 
-    def _gen_prompt(self):
+    # I/O #####################################################################
+    def write_warnings(self):
+        for warning in self.warnings:
+            self.write(color('WARNING: {}\n'.format(warning), fg='yellow'))
+
+    def write_banner(self):
+        self.write(self.banner)
+
+    def gen_prompt(self):
         prompt_color = 'green'
 
         if self.exit_code != 0:
             prompt_color = 'red'
 
         ps1 = color(
-            self.ps1,
-            color=prompt_color,
+            self.prompt_prefix + self.ps1,
+            fg=prompt_color,
             style='bright',
         )
 
         ps2 = color(
-            self.ps2,
-            color=prompt_color,
+            self.prompt_prefix + self.ps2,
+            fg=prompt_color,
             style='normal',
         )
 
         if not self.line_buffer:
             return ps1
 
-        if self._validate_line_buffer():
+        if self.validate_line_buffer():
             return ps1
 
         return ps2
@@ -159,11 +193,11 @@ class Repl:
     def interact(self):
         while True:
             try:
-                self.line_buffer += input(self._gen_prompt()) + '\n'
+                self.line_buffer += input(self.gen_prompt()) + '\n'
 
                 # handle empty lines
                 if not self.line_buffer.strip():
-                    self.clear_line_buffer()
+                    self.handle_empty_line()
 
                     continue
 
@@ -185,11 +219,11 @@ class Repl:
                 continue
 
             # multi line
-            if not self._validate_line_buffer():
+            if not self.validate_line_buffer():
                 continue
 
             self.add_history(self.line_buffer.strip())
-            self.run()
+            self.run(self.line_buffer)
             self.clear_line_buffer()
 
     # application code ########################################################
@@ -197,13 +231,26 @@ class Repl:
         print(string, end='')
 
     def setup(self):
-        self.write(self.banner)
-
-        for warning in self.warnings:
-            self.write(color('WARNING: {}\n'.format(warning), color='yellow'))
+        self.write_banner()
+        self.write_warnings()
 
     def complete(self, text, state):
-        return self.python_runtime.complete(text, state)
+        rl_line_buffer = readline.get_line_buffer()
+
+        if rl_line_buffer.startswith('%'):
+            rl_line_buffer = rl_line_buffer[1:]
+
+            return self.shell_runtime.complete(
+                text=text,
+                state=state,
+                line_buffer=rl_line_buffer,
+            )
+
+        else:
+            return self.python_runtime.complete(text, state)
+
+    def handle_empty_line(self):
+        self.clear_line_buffer()
 
     def handle_ctrl_d(self):
         self.clear_line_buffer()
@@ -230,15 +277,17 @@ class Repl:
         self.write('^C\n')
         self.exit_code = 1
 
-    def run(self):
+    def run(self, command):
         try:
-            if self.line_buffer.startswith('%'):
-                self.exit_code = self.shell_runtime.run(self.line_buffer)
+            if command.startswith('%'):
+                self.exit_code = self.shell_runtime.run(command)
 
             else:
-                self.exit_code = self.python_runtime.run(self.line_buffer)
+                self.exit_code = self.python_runtime.run(command)
 
         except Exception as exception:
             self.exit_code = 1
 
             self.write_exception(exception, prefix='rlpython: ')
+
+        return self.exit_code
