@@ -1,9 +1,11 @@
 import shlex
 
 from rlpython.utils.argument_parser import ReplArgumentParserError
+from rlpython.commands.threads import ThreadsCommand
 from rlpython.commands.edit import EditCommand
 
 DEFAULT_COMMANDS = [
+    ThreadsCommand,
     EditCommand,
 ]
 
@@ -16,12 +18,26 @@ class ShellRuntime:
         for command in DEFAULT_COMMANDS:
             self.install_command(command)
 
+    def shutdown(self):
+        pass
+
     def install_command(self, command_class, name=''):
         command = command_class()
         name = name or command.NAME
 
         self.commands[name] = command
 
+    def complete(self, text, state, line_buffer):
+        candidates = []
+
+        for name in sorted(self.commands.keys()):
+            if name.startswith(text):
+                candidates.append(name)
+
+        candidates.append(None)
+
+        return candidates[state]
+
     def validate_source(self, raw_source):
         try:
             shlex.split(raw_source)
